/*****************************************************************************
 *
 * HYPERGRAPH.C - Nagios Network Status Map CGI
 *
 * Copyright (c) 2006 Hugo Hallqvist (hugo@op5.se)
 * Last Modified: 2006-05-03
 *
 * Description:
 *
 * This CGI will create a xml file of all hosts that are being monitored, suitable
 * for being interpreted by the hypergraph applet (http://hypergraph.sf.net).
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../include/config.h"
#include "../include/common.h"
#include "../include/objects.h"
#include "../include/statusdata.h"

#include "../include/cgiutils.h"
#include "../include/getcgi.h"
#include "../include/cgiauth.h"

extern int             refresh_rate;

/*#define DEBUG*/

#define UNKNOWN_ICON_IMAGE    "unknown.gif"

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_stylesheets_path[MAX_FILENAME_LENGTH];

extern host *host_list;
extern hostgroup *hostgroup_list;
extern service *service_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;

#define CREATE_HTML	0
#define CREATE_XML	1

void document_header(int);
void document_footer(void);
int process_cgivars(void);

void display_page_header(void);
void display_map(void);
void draw_hosts(void);
void find_eligible_hosts(void);

extern char url_images_path[MAX_FILENAME_LENGTH];

authdata current_authdata;

int create_type = CREATE_HTML;
int display_header = TRUE;
char *host_name = NULL;

extern time_t program_start;

int main(int argc, char **argv) {
	int result;

	/* reset internal variables */
	reset_cgi_vars();

	/* read the CGI configuration file */
	result = read_cgi_config_file(get_cgi_config_location());
	if(result == ERROR) {
		document_header(FALSE);
		if(create_type == CREATE_HTML)
			cgi_config_file_error(get_cgi_config_location());
		document_footer();
		return ERROR;
		}

	/* get the arguments passed in the URL */
	process_cgivars();

	/* read the main configuration file */
	result = read_main_config_file(main_config_file);
	if(result == ERROR) {
		document_header(FALSE);
		if(create_type == CREATE_HTML)
			main_config_file_error(main_config_file);
		document_footer();
		return ERROR;
		}

	/* read all object configuration data */
	result = read_all_object_configuration_data(main_config_file, READ_ALL_OBJECT_DATA);
	if(result == ERROR) {
		document_header(FALSE);
		if(create_type == CREATE_HTML)
			object_data_error();
		document_footer();
		return ERROR;
		}

	/* read all status data */
	result = read_all_status_data(get_cgi_config_location(), READ_ALL_STATUS_DATA);
	if(result == ERROR) {
		document_header(FALSE);
		if(create_type == CREATE_HTML)
			status_data_error();
		document_footer();
		free_memory();
		return ERROR;
		}


	document_header(TRUE);

	/* get authentication information */
	get_authentication_information(&current_authdata);

	/* display the network map... */
	display_map();

	document_footer();

	/* free all allocated memory */
	free_memory();

	return OK;
	}



void document_header(int use_stylesheet) {
	char date_time[MAX_DATETIME_LENGTH];
	time_t current_time;
	time_t expire_time;

	if(create_type == CREATE_HTML) {
		printf("Cache-Control: no-store\n");
		printf("Pragma: no-cache\n");
		/* Since the hypergraph map is primarily used to
		 * locate hosts on a large network rather than
		 * finding out what's gone wrong, it doesn't
		 * make sense to refresh it.
		 */
		/* printf("Refresh: %d\n",refresh_rate); */

		time(&current_time);
		get_time_string(&current_time, date_time, sizeof(date_time), HTTP_DATE_TIME);
		printf("Last-Modified: %s\n", date_time);

		expire_time = 0L;
		get_time_string(&expire_time, date_time, sizeof(date_time), HTTP_DATE_TIME);
		printf("Expires: %s\n", date_time);

		printf("Content-Type: text/html\n\n");

		printf("<html>\n");
		printf("<head>\n");
		printf("<title>\n");
		printf("Hyper Map\n");
		printf("</title>\n");

		if(use_stylesheet == TRUE) {
			printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n", url_stylesheets_path, COMMON_CSS);
			printf("<LINK REL='stylesheet' TYPE='text/css' HREF='%s%s'>\n", url_stylesheets_path, STATUSMAP_CSS);
			}

		printf("</head>\n");

		printf("<body CLASS='statusmap' name='mappage' id='mappage'>\n");

		/* include user SSI header */
		include_ssi_files(STATUSMAP_CGI, SSI_HEADER);

		printf("<div id=\"popup\" style=\"position:absolute; z-index:1; visibility: hidden\"></div>\n");
		}

	else {
		printf("Cache-Control: no-store\n");
		printf("Pragma: no-cache\n");

		time(&current_time);
		get_time_string(&current_time, date_time, sizeof(date_time), HTTP_DATE_TIME);
		printf("Last-Modified: %s\n", date_time);

		expire_time = (time_t)0L;
		get_time_string(&expire_time, date_time, sizeof(date_time), HTTP_DATE_TIME);
		printf("Expires: %s\n", date_time);
		printf("Content-Type: text/xml\n\n");

		printf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		printf("<!DOCTYPE GraphXML SYSTEM \"/monitor/GraphXML.dtd\">\n");
		printf("<GraphXML xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n");
		printf("<graph>\n");
		printf("<style>\n");
		printf("<line tag=\"node\" class=\"main\" colour=\"#ffffff\"/>\n");
		printf("<line tag=\"node\" class=\"child_node\" colour=\"blue\"/>\n");
		printf("<line tag=\"node\" class=\"nagios_node\" colour=\"red\"/>\n");
		printf("<line tag=\"node\" class=\"relation_node\" colour=\"green\"/>\n");
		printf("<line tag=\"node\" class=\"hs0\" colour=\"#33FF00\"/>\n");
		printf("<line tag=\"node\" class=\"hs1\" colour=\"#F83838\"/>\n");
		printf("<line tag=\"node\" class=\"hs2\" colour=\"#F83838\"/>\n");
		printf("<line tag=\"edge\" class=\"secondary_edge\" colour=\"blue\" linestyle=\"dashed\"/>\n");
		printf("</style>\n");
		}

	return;
	}


void document_footer(void) {

	if(create_type == CREATE_XML) {
		printf("</graph></GraphXML>\n");
		}
	if(create_type == CREATE_HTML) {

		/* include user SSI footer */
		include_ssi_files(STATUSMAP_CGI, SSI_FOOTER);

		printf("</body>\n");
		printf("</html>\n");
		}

	return;
	}



int process_cgivars(void) {
	char **variables;
	int error = FALSE;
	int x;

	variables = getcgivars();

	for(x = 0; variables[x] != NULL; x++) {

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x]) >= MAX_INPUT_BUFFER - 1) {
			x++;
			continue;
			}

		/* we found the host argument */
		else if(!strcmp(variables[x], "host")) {
			x++;
			if(variables[x] == NULL) {
				error = TRUE;
				break;
				}

			host_name = (char *)malloc(strlen(variables[x]) + 1);
			if(host_name != NULL)
				strcpy(host_name, variables[x]);

			}

		/* we found the xml creation option */
		else if(!strcmp(variables[x], "createxml")) {
			create_type = CREATE_XML;
			}
		}
	/* free memory allocated to the CGI variables */
	free_cgivars(variables);

	return error;
	}



/* top of page */
void display_page_header(void) {
	char temp_buffer[MAX_INPUT_BUFFER];

	if(create_type != CREATE_HTML)
		return;

	if(display_header == TRUE) {

		/* begin top table */
		printf("<table border=0 width=100%% cellspacing=0 cellpadding=0>\n");
		printf("<tr>\n");

		/* left column of the first row */
		printf("<td align=left valign=top>\n");

		snprintf(temp_buffer, sizeof(temp_buffer) - 1, "Hypergraph Map For All Hosts");
		temp_buffer[sizeof(temp_buffer) - 1] = '\x0';
		display_info_table(temp_buffer, TRUE, &current_authdata);

		printf("<TABLE BORDER=1 CELLPADDING=0 CELLSPACING=0 CLASS='linkBox'>\n");
		printf("<TR><TD CLASS='linkBox'>\n");

		printf("<a href='%s?host=all'>View Status Detail For All Hosts</a><BR>\n", STATUS_CGI);
		printf("<a href='%s?hostgroup=all'>View Status Overview For All Hosts</a>\n", STATUS_CGI);

		printf("</TD></TR>\n");
		printf("</TABLE>\n");

		printf("</td>\n");



		/* center column of top row */
		printf("<td align=center valign=center>\n");
		printf("</td>\n");



		/* right hand column of top row */
		printf("<td align=right valign=top>\n");
		printf("</td>\n");

		/* end of top table */
		printf("</tr>\n");
		printf("</table>\n");
		}


	return;
	}



/* top-level map generation... */
void display_map(void) {
	find_eligible_hosts();

	/* display page header */
	display_page_header();

	/* User needs to have rights to all hosts and services in order to view the map.  */
	if(!(is_authorized_for_all_hosts(& current_authdata) && is_authorized_for_all_services(& current_authdata))) {
		if(create_type == CREATE_HTML) {
			printf("<P><DIV ALIGN=CENTER>\n");
			printf("<DIV CLASS='errorMessage'>It appears as though you do not have permission to view information for all hosts and all services.</DIV>\n");
			printf("<DIV CLASS='errorDescription'>If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
			printf("and check the authorization options in your CGI configuration file.</DIV>\n");
			printf("</DIV></P>");
			}

		return;
		}

	if(create_type == CREATE_HTML) {
		printf("<A name=\"graph-section\"></A>\n");
		printf("<APPLET code=\"hypergraph.applications.hexplorer.HExplorerApplet.class\" archive=\"%shyperapplet.jar\" width=\"100%%\" height=\"100%%\" align=\"baseline\">\n",
		       url_html_path);
		printf("<PARAM name=\"file\" value=\"hypergraph.cgi?createxml\">\n");
		printf("<PARAM name=\"properties\" value=\"%snagios.prop\">\n", url_html_path);
		if(host_name != NULL) {
			printf("<PARAM name=\"center\" value=\"%s\">\n", host_name);
			}
		printf("</PARAM>\n");
		printf("</APPLET>\n");
		return;
		}
	else {
		draw_hosts();
		}

	return;
	}



/* finds hosts that can be drawn in the canvas area */
void find_eligible_hosts(void) {
	int total_eligible_hosts = 0;
	host *temp_host;

	/* Nagios 3 merged host-/service- objects and their respective
	 * extinfo objects, so we just walk the hostlist and print it as
	 * we go
	 */
	for(temp_host = host_list; temp_host != NULL; temp_host = temp_host->next) {
		/* see if the user is authorized to view the host */
		if(is_authorized_for_host(temp_host, &current_authdata) == FALSE) {
			temp_host->should_be_drawn = FALSE;
			continue;
			}

		/* all checks passed, so we can draw the host! */
		temp_host->should_be_drawn = TRUE;
		total_eligible_hosts++;
		}

	return;
	}

char *host_status_texts[] = {"UP", "DOWN", "UNREACHABLE", "PENDING", "UNKNOWN"};
#define BUFSIZE 1000
void draw_hosts(void) {
	host *temp_host;
	hostsmember *parent;
	int host_state;
	char *host_name_html, *host_name_url, *title_str;
	char buffer[BUFSIZE];

	// The monitor host
	printf("<node class=\"nagios_node\" name=\"%u\"><label>%s</label></node>\n", 0, "Monitor Process");

	/* draw all hosts... */
	for(temp_host = host_list; temp_host != NULL; temp_host = temp_host->next) {
		hoststatus *temp_hoststatus;

		/* skip hosts that should not be drawn */
		// TODO: Fix this
		/*
		if(temp_hostextinfo->should_be_drawn==FALSE)
			continue;
		*/

		host_state = 4;
		temp_hoststatus = find_hoststatus(temp_host->name);
		if(temp_hoststatus != NULL) {
			if(temp_hoststatus->status == HOST_DOWN)
				host_state = 1;
			else if(temp_hoststatus->status == HOST_UNREACHABLE)
				host_state = 2;
			else if(temp_hoststatus->status == HOST_UP)
				host_state = 0;
			else if(temp_hoststatus->status == HOST_PENDING)
				host_state = 3;
			}
		// TODO: add xlink:title with description of varios things (status, description, number of host block (outages))
		host_name_url = strdup(url_encode(temp_host->name));
		//host_name_html = strdup(html_encode(temp_host->name));
		host_name_html = strdup(temp_host->name);
		snprintf(buffer, BUFSIZE, "Host: %s Status: %s (Click for detail)", temp_host->name, host_status_texts[host_state]);
		//title_str = strdup(html_encode(buffer));
		title_str = strdup(buffer);
		printf("<node class=\"hs%u\" name=\"%s\"><label>%s</label><dataref><ref xlink:show=\"replace\" xlink:href=\"status.cgi?host=%s\" xlink:title=\"%s\"/></dataref></node>\n",
		       host_state, host_name_html, host_name_html, host_name_url, title_str);
		free(title_str);
		free(host_name_url);
		free(host_name_html);
		}

	for(temp_host = host_list; temp_host != NULL; temp_host = temp_host->next) {
		/* skip hosts that should not be drawn */
		// TODO: Fix this
		/*
		if(temp_hostextinfo->should_be_drawn==FALSE)
			continue;
		*/

		host_name_html = strdup(html_encode(temp_host->name, FALSE));
		if(!temp_host->parent_hosts) {
			printf("<edge source=\"%u\" target=\"%s\"/>\n", 0, host_name_html);
			}
		// Edges to all parents of this host
		for(parent = temp_host->parent_hosts; parent; parent = parent->next) {
			printf("<edge source=\"%s\" target=\"%s\"/>\n", html_encode(parent->host_name, FALSE), host_name_html);
			}
		}

	}
