#include "../include/cgiutils.h"
#include "op5tools.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

#ifndef ARRAY_SIZE
# define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
#endif

#define OP5TOOL_ID(flags) (flags & 0xff)

#ifdef TEST_OP5TOOLS
# define url_encode(s) (s)
#endif

struct op5tool {
	const char *base_url;
	const char *path;
	const char *href_text;
	const char *icon_url;
	const char *what;
	};

static struct op5tool tools[] = {
		{
		"/monitor/op5/nacoma/edit.php",
		"/opt/monitor/op5/nacoma/edit.php",
		"op5 Monitor Webconfiguration",
		"/monitor/images/op5tools/webconfig.png",
		"Configure this %s using op5 Monitor webconfiguration tool",
		},
	};

struct op5tool *op5tool_exists(int flags) {
	struct stat st;

	if(OP5TOOL_ID(flags) > ARRAY_SIZE(tools))
		return NULL;
	if(stat(tools[OP5TOOL_ID(flags)].path, &st) < 0 && errno == ENOENT)
		return NULL;

	return &tools[OP5TOOL_ID(flags)];
	}

static char url[8192];
static const char *op5tool_url(char *hst, char *srv, struct op5tool *tool) {
	int len;

	if(!hst)
		return NULL;

	len = snprintf(url, sizeof(url), "%s?obj_type=%s&host=%s",
	               tool->base_url, srv ? "service" : "host", url_encode(hst));
	if(srv)
		snprintf(url + len, sizeof(url) - len, "&service=%s", url_encode(srv));

	return url;
	}

void op5tool_print_link(char *hst, char *srv, int flags) {
	struct op5tool *tool;

	tool = op5tool_exists(flags);
	if(!tool)
		return;

	printf("<a href='%s' title='", op5tool_url(hst, srv, tool));
	printf(tool->what, srv ? "service" : "host");
	printf("'>\n");
	if(tool->icon_url)
		printf("<img src='%s'>", tool->icon_url);
	else if(tool->href_text)
		printf("%s", tool->href_text);
	printf("</a>");
	}

#ifdef TEST_OP5TOOLS
int main(int argc, char **argv) {
	struct op5tool *tool;

	tool = op5tool_exists(OP5TOOL_WEBCONFIG);
	if(tool)
		printf("webconfig tool exists\n");
	else
		return 1;

	printf("service url: %s\n", op5tool_url("foohost", "barservice", tool));
	printf("host url   : %s\n", op5tool_url("foohost", NULL, tool));
	op5tool_print_link("foohost", "barservice", OP5TOOL_WEBCONFIG);
	putchar('\n');
	return 0;
	}
#endif
