#include "../include/cgiutils.h"
#include "pnp.h"
#include <stdlib.h>
#include <stdarg.h>
#include <limits.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef TEST_PNP
# define url_encode(str) str
#endif

static char *pnp_rrd_path = "/opt/monitor/op5/pnp/perfdata";
static char *pnp_url = "/monitor/op5/pnp";
static char *logo_url = "/monitor/images/logos/pnp_icon.png";

void pnp_set_option(const char *key, const char *value) {
	if(!strcmp(key, "pnp_rrd_path")) {
		pnp_rrd_path = strdup(value);
		return;
		}
	if(!strcmp(key, "pnp_url")) {
		pnp_url = strdup(value);
		return;
		}
	}

static const int pnp_cleanup(char *dest, const char *src) {
	int i;

	for(i = 0; i < strlen(src); i++) {
		switch(src[i]) {
			case ' ':
			case ':':
			case '/':
			case '\\':
				dest[i] = '_';
				break;
			default:
				dest[i] = src[i];
				break;
			}
		}

	return i;
	}

/*
 * it's up to the caller to make sure *path
 * is big enough to store the entire path
 */
static int pnp_base_path(char *path, const char *hst, const char *srv) {
	int pos;

	if(!pnp_rrd_path)
		return -1;

	pos = strlen(pnp_rrd_path);
	memcpy(path, pnp_rrd_path, strlen(pnp_rrd_path));
	path[pos++] = '/';
	pos += pnp_cleanup(path + pos, hst);
	path[pos++] = '/';
	pos += pnp_cleanup(path + pos, srv ? srv : "_HOST_");

	return pos;
	}

static int pnp_mkpath(char *path, const char *hst, const char *srv, const char *suffix) {
	int pos;
	size_t suffix_len = strlen(suffix);

	pos = pnp_base_path(path, hst, srv);
	if(pos < 0)
		return pos;

	/* include nul-terminator */
	memcpy(path + pos, suffix, suffix_len + 1);
	return pos + suffix_len;
	}

#define pnp_xml_path(path, hst, srv) pnp_mkpath(path, hst, srv, ".xml")
#define pnp_rrd_path(path, hst, srv) pnp_mkpath(path, hst, srv, ".rrd")

int pnp_has_graph(const char *hst, const char *srv) {
	char path[PATH_MAX];

	if(!pnp_rrd_path)
		return 0;

	if(pnp_xml_path(path, hst, srv) < 0)
		return 0;
	if(access(path, R_OK) < 0)
		return 0;

	if(pnp_rrd_path(path, hst, srv) < 0)
		return 0;
	if(access(path, R_OK) < 0)
		return 0;

	return 1;
	}

static char *pnp_graph_url(char *hst, char *srv) {
	char *url = NULL;

	if(!pnp_url)
		return NULL;

	if(srv) {
		asprintf(&url, "%s/index.php?host=%s&srv=%s",
		         pnp_url, url_encode(hst), url_encode(srv));
		}
	else {
		asprintf(&url, "%s/index.php?host=%s", pnp_url, url_encode(hst));
		}

	return url;
	}

void pnp_print_graph_link(char *hst, char *srv) {
	char *url;

	if(!pnp_url || !pnp_has_graph(hst, srv))
		return;

	url = pnp_graph_url(hst, srv);
	printf("<a href='%s'>", url);
	printf("<img src='%s' border='0' width='%d' height='%d' ",
	       logo_url, STATUS_ICON_WIDTH, STATUS_ICON_HEIGHT);
	printf(" alt='Show performance graph' title='Show performance graph'>");
	printf("</a>");
	free(url);
	}

#ifdef TEST_PNP
int main(int argc, char **argv) {
	char *arg;
	int i;
	char path[PATH_MAX];

	for(i = 1; i < argc; i++) {
		char *arg = strdup(argv[i]);
		char *sc = strchr(arg, ';');
		int len;

		if(sc)
			*sc++ = '\0';

		len = pnp_xml_path(path, arg, sc);
		if(pnp_has_graph(arg, sc))
			printf("has graph");
		else
			printf("no graph");
		printf(": %s\n", path);
		printf("%s\n", pnp_graph_url(arg, sc));
		}

	return 0;
	}
#endif
